@@ -2,7 +2,7 @@ import logging
 import math
 import re
 from abc import ABC, abstractmethod
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import Iterable, List, NamedTuple, Optional
 
 from lxml import etree
@@ -34,14 +34,31 @@ DEFAULT_DOCUMENT_FEATURES_CONTEXT = DocumentFeaturesContext()
 @dataclass
 class LayoutModelData:
     data_line: str
-    layout_line: Optional[LayoutLine] = None
-    layout_token: Optional[LayoutToken] = None
+    layout_line: Optional[LayoutLine] = field(repr=False, default=None)
+    layout_token: Optional[LayoutToken] = field(repr=False, default=None)
 
     @property
     def label_token_text(self):
         return self.data_line.split(' ', maxsplit=1)[0]
 
 
+@dataclass
+class LabeledLayoutModelData(LayoutModelData):
+    label: Optional[str] = None
+
+    @staticmethod
+    def from_model_data(
+        model_data: LayoutModelData,
+        label: Optional[str] = None
+    ) -> 'LabeledLayoutModelData':
+        return LabeledLayoutModelData(
+            data_line=model_data.data_line,
+            layout_line=model_data.layout_line,
+            layout_token=model_data.layout_token,
+            label=label
+        )
+
+
 class ModelDataGenerator(ABC):
     def iter_data_lines_for_xml_root(
         self,
@@ -695,6 +712,9 @@ class ContextAwareLayoutTokenModelDataGenerator(ModelDataGenerator):
             for block in layout_document.iter_all_blocks()
             for line in block.lines
         }
+        if not concatenated_line_tokens_length_by_line_id:
+            LOGGER.debug('empty layout document')
+            return
         max_concatenated_line_tokens_length = max(
             concatenated_line_tokens_length_by_line_id.values()
         )
