@@ -3,6 +3,7 @@ import logging
 from sciencebeam_parser.models.data import (
     DocumentFeaturesContext
 )
+from sciencebeam_parser.models.figure.training_data import FigureTeiTrainingDataGenerator
 from sciencebeam_parser.models.model import Model
 from sciencebeam_parser.models.fulltext.data import FullTextDataGenerator
 from sciencebeam_parser.models.figure.extract import FigureSemanticExtractor
@@ -22,3 +23,6 @@ class FigureModel(Model):
 
     def get_semantic_extractor(self) -> FigureSemanticExtractor:
         return FigureSemanticExtractor()
+
+    def get_tei_training_data_generator(self) -> FigureTeiTrainingDataGenerator:
+        return FigureTeiTrainingDataGenerator()
