@@ -6,6 +6,9 @@ from sciencebeam_parser.models.data import (
 from sciencebeam_parser.models.model import Model
 from sciencebeam_parser.models.fulltext.data import FullTextDataGenerator
 from sciencebeam_parser.models.table.extract import TableSemanticExtractor
+from sciencebeam_parser.models.table.training_data import (
+    TableTeiTrainingDataGenerator
+)
 
 
 LOGGER = logging.getLogger(__name__)
@@ -22,3 +25,6 @@ class TableModel(Model):
 
     def get_semantic_extractor(self) -> TableSemanticExtractor:
         return TableSemanticExtractor()
+
+    def get_tei_training_data_generator(self) -> TableTeiTrainingDataGenerator:
+        return TableTeiTrainingDataGenerator()
